//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/DetectorEditor.cpp
//! @brief     Implements class DetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/DetectorEditor.h"
#include "GUI/Model/Detector/DetectorItem.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Support/Style/GroupBoxCollapser.h"
#include "GUI/View/Device/DetectorAlignmentForm.h"
#include "GUI/View/Device/ResolutionForm.h"
#include "GUI/View/Numeric/DoubleSpinBox.h"
#include "GUI/View/Numeric/NumWidgetUtil.h"
#include "GUI/View/Numeric/SafeSpinBox.h"
#include <QFormLayout>

#include "GUI/Support/Style/ApplicationSettings.h"
#include <QLabel>

void installGroup(QGroupBox* groupBox)
{
    auto* label = new QLabel(groupBox);
    label->setObjectName("GroupBoxToggler");
    if (appSettings->currentStyle() == ApplicationSettings::Style::native)
        label->setStyleSheet(
            "QLabel { border: none; text-align: left; font: bold; padding: 5px}");
    label->setText(groupBox->title());
    label->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

    auto* contentArea = new QWidget(groupBox);
    contentArea->setObjectName("ContentArea");
    contentArea->setLayout(groupBox->layout());

    auto* mainLayout = new QVBoxLayout;
    mainLayout->setSpacing(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setMenuBar(label);
    mainLayout->addWidget(contentArea);

    groupBox->setTitle(""); // title has been read above
    groupBox->setLayout(mainLayout); // overwrites layout that has been read above
}

DetectorEditor::DetectorEditor(QWidget* parent, GISASInstrumentItem* instrItem)
    : QGroupBox("Detector parameters", parent)
{
    ASSERT(instrItem);

    auto* formLayout = new QFormLayout(this);

    auto* collapser = GroupBoxCollapser::installIntoGroupBox(this);
    collapser->setExpanded(instrItem->isExpandDetector());
    connect(collapser, &GroupBoxCollapser::toggled, this,
            [instrItem](bool b) { instrItem->setExpandDetector(b); });

    DetectorItem* detectorItem = instrItem->detectorItem();
    ASSERT(detectorItem);

    auto* grid = new QGridLayout;
    grid->setColumnStretch(0, 1);
    grid->setColumnStretch(1, 1);
    grid->setColumnStretch(2, 1);

    // -- x-axis controls
    auto* xAxisGroupBox = new QGroupBox("X axis", this);
    auto* xAxisFormLayout = new QFormLayout(xAxisGroupBox);

    auto* xAxisNbinsSpinBox = new SafeSpinBox;
    xAxisNbinsSpinBox->setRange(1, 65536);
    xAxisNbinsSpinBox->setValue(detectorItem->xSize());
    xAxisFormLayout->addRow("# bins:", xAxisNbinsSpinBox);

    auto* widthSpinBox = new DoubleSpinBox(detectorItem->width());
    xAxisFormLayout->addRow("Width [mm]:", widthSpinBox);

    connect(xAxisNbinsSpinBox, qOverload<int>(&QSpinBox::valueChanged),
            [this, detectorItem](int newValue) {
                detectorItem->setXSize(newValue);
                emit dataChanged();
            });

    connect(widthSpinBox, qOverload<double>(&DoubleSpinBox::baseValueChanged),
            [this, detectorItem](double newValue) {
                detectorItem->setWidth(newValue);
                emit dataChanged();
            });
    grid->addWidget(xAxisGroupBox, 1, 0);
    installGroup(xAxisGroupBox);


    // -- y-axis controls
    auto* yAxisGroupBox = new QGroupBox("Y axis", this);
    auto* yAxisFormLayout = new QFormLayout(yAxisGroupBox);

    auto* yAxisNbinsSpinBox = new SafeSpinBox;
    yAxisNbinsSpinBox->setRange(1, 65536);
    yAxisNbinsSpinBox->setValue(detectorItem->ySize());
    yAxisFormLayout->addRow("# bins:", yAxisNbinsSpinBox);

    auto* heightSpinBox = new DoubleSpinBox(detectorItem->height());
    yAxisFormLayout->addRow("Height [mm]:", heightSpinBox);

    connect(yAxisNbinsSpinBox, qOverload<int>(&QSpinBox::valueChanged),
            [this, detectorItem](int newValue) {
                detectorItem->setYSize(newValue);
                emit dataChanged();
            });

    connect(heightSpinBox, qOverload<double>(&DoubleSpinBox::baseValueChanged),
            [this, detectorItem](double newValue) {
                detectorItem->setHeight(newValue);
                emit dataChanged();
            });
    grid->addWidget(yAxisGroupBox, 1, 1);

    auto* collapserY = GroupBoxCollapser::installIntoGroupBox(yAxisGroupBox);
    collapserY->setExpanded(detectorItem->isExpandYaxis());
    connect(collapserY, &GroupBoxCollapser::toggled, this,
            [detectorItem](bool b) { detectorItem->setExpandYaxis(b); });

    // -- resolution function
    auto* resolutionForm = new ResolutionForm(ResolutionForm::Millimeter, this, detectorItem);
    grid->addWidget(resolutionForm, 1, 2);

    connect(resolutionForm, &ResolutionForm::dataChanged, this, &DetectorEditor::dataChanged);

    // alignment selector editors
    auto* alignmentForm = new DetectorAlignmentForm(this, detectorItem);
    grid->addWidget(alignmentForm, 2, 0, 1, 3);

    connect(alignmentForm, &DetectorAlignmentForm::dataChanged, this, &DetectorEditor::dataChanged);

    formLayout->addRow(grid);
}
